@@ -19,31 +19,33 @@ import java.io.FileReader;
 import java.io.FileWriter;
 import java.io.IOException;
 import java.io.PrintWriter;
+import java.nio.ByteBuffer;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Calendar;
+import java.util.Collection;
 import java.util.Date;
 import java.util.List;
-import java.util.Random;
 import java.util.TimeZone;
 
-import junit.framework.TestCase;
-
-import com.sun.jna.Memory;
 import com.sun.jna.Native;
 import com.sun.jna.NativeMappedConverter;
 import com.sun.jna.Platform;
 import com.sun.jna.Pointer;
+import com.sun.jna.platform.win32.BaseTSD.SIZE_T;
 import com.sun.jna.platform.win32.WinBase.MEMORYSTATUSEX;
 import com.sun.jna.platform.win32.WinBase.SYSTEM_INFO;
 import com.sun.jna.platform.win32.WinDef.DWORD;
 import com.sun.jna.platform.win32.WinDef.HWND;
 import com.sun.jna.platform.win32.WinNT.HANDLE;
 import com.sun.jna.platform.win32.WinNT.HANDLEByReference;
-import com.sun.jna.platform.win32.WinNT.LARGE_INTEGER;
+import com.sun.jna.platform.win32.WinNT.MEMORY_BASIC_INFORMATION;
 import com.sun.jna.platform.win32.WinNT.OSVERSIONINFO;
 import com.sun.jna.platform.win32.WinNT.OSVERSIONINFOEX;
 import com.sun.jna.ptr.IntByReference;
 
+import junit.framework.TestCase;
+
 public class Kernel32Test extends TestCase {
 
     public static void main(String[] args) {
@@ -56,6 +58,21 @@ public class Kernel32Test extends TestCase {
         junit.textui.TestRunner.run(Kernel32Test.class);
     }
 
+    // see https://github.com/twall/jna/issues/482
+    public void testNoDuplicateMethodsNames() {
+        Collection<String> dupSet = AbstractWin32TestSupport.detectDuplicateMethods(Kernel32.class);
+        if (dupSet.size() > 0) {
+            for (String name : new String[] {
+                    // has 2 overloads by design since the API accepts both OSVERSIONINFO and OSVERSIONINFOEX
+                    "GetVersionEx"
+                }) {
+                dupSet.remove(name);
+            }
+        }
+
+        assertTrue("Duplicate methods found: " + dupSet, dupSet.isEmpty());
+    }
+
     public void testGetDriveType() {
         if (!Platform.isWindows()) return;
 
@@ -77,6 +94,93 @@ public class Kernel32Test extends TestCase {
                      cal.get(Calendar.YEAR), time.wYear);
     }
 
+    public void testSetSystemTime() {
+        Kernel32 kernel = Kernel32.INSTANCE;
+        WinBase.SYSTEMTIME time = new WinBase.SYSTEMTIME();
+        kernel.GetSystemTime(time);
+        try {
+            WinBase.SYSTEMTIME expected = new WinBase.SYSTEMTIME();
+            expected.wYear = time.wYear;
+            expected.wMonth = time.wMonth;
+            expected.wDay = time.wDay;
+            expected.wHour = time.wHour;
+            expected.wMinute = time.wMinute;
+            expected.wSecond = time.wSecond;
+            expected.wMilliseconds = time.wMilliseconds;
+
+            if (expected.wHour > 0) {
+                expected.wHour--;
+            } else {
+                expected.wHour++;
+            }
+
+            if (assertTimeSettingOperationSucceeded("SetSystemTime", kernel.SetSystemTime(expected))) {
+                WinBase.SYSTEMTIME actual = new WinBase.SYSTEMTIME();
+                kernel.GetSystemTime(actual);
+                assertEquals("Mismatched hour value", expected.wHour, actual.wHour);
+            }
+        } finally {
+            assertTimeSettingOperationSucceeded("Restore original system time", kernel.SetSystemTime(time));
+        }
+    }
+
+    public void testSetLocaltime() {
+        Kernel32 kernel = Kernel32.INSTANCE;
+        WinBase.SYSTEMTIME time = new WinBase.SYSTEMTIME();
+        kernel.GetLocalTime(time);
+        try {
+            WinBase.SYSTEMTIME expected = new WinBase.SYSTEMTIME();
+            expected.wYear = time.wYear;
+            expected.wMonth = time.wMonth;
+            expected.wDay = time.wDay;
+            expected.wHour = time.wHour;
+            expected.wMinute = time.wMinute;
+            expected.wSecond = time.wSecond;
+            expected.wMilliseconds = time.wMilliseconds;
+
+            if (expected.wHour > 0) {
+                expected.wHour--;
+            } else {
+                expected.wHour++;
+            }
+
+            if (assertTimeSettingOperationSucceeded("SetLocalTime", kernel.SetLocalTime(expected))) {
+                WinBase.SYSTEMTIME actual = new WinBase.SYSTEMTIME();
+                kernel.GetLocalTime(actual);
+                assertEquals("Mismatched hour value", expected.wHour, actual.wHour);
+            }
+        } finally {
+            assertTimeSettingOperationSucceeded("Restore local time", kernel.SetLocalTime(time));
+        }
+    }
+
+    private static boolean assertTimeSettingOperationSucceeded(String message, boolean result) {
+        if (result) {
+            return result;
+        }
+
+        int hr=Kernel32.INSTANCE.GetLastError();
+        /*
+         * Check special error in case the user running the test isn't allowed
+         * to change the time. This can happen for hosts that are managed
+         * by some central administrator using an automated time setting mechanism.
+         * In such cases, the user running the tests might not have admin
+         * privileges and it may be too much to ask to have them just for running
+         * this JNA API test...
+         */
+        if (hr == WinError.ERROR_PRIVILEGE_NOT_HELD) {
+            return false; // don't fail the test, but signal the failure
+        }
+
+        if (hr != WinError.ERROR_SUCCESS) {
+            fail(message + " failed: hr=" + hr);
+        } else {
+            fail(message + " unknown failure reason code");
+        }
+
+        return false;
+    }
+
     public void testGetLastError() {
         Kernel32 kernel = Kernel32.INSTANCE;
         int ERRCODE  = 8;
@@ -109,6 +213,22 @@ public class Kernel32Test extends TestCase {
     	assertTrue(Kernel32.INSTANCE.GetComputerName(buffer, lpnSize));
     }
 
+    public void testGetComputerNameExSameAsGetComputerName() {
+    	IntByReference lpnSize = new IntByReference(0);
+    	char buffer[] = new char[WinBase.MAX_COMPUTERNAME_LENGTH + 1];
+    	lpnSize.setValue(buffer.length);
+    	assertTrue("Failed to retrieve expected computer name", Kernel32.INSTANCE.GetComputerName(buffer, lpnSize));
+        String expected = Native.toString(buffer);
+
+        // reset
+    	lpnSize.setValue(buffer.length);
+        Arrays.fill(buffer, '\0');
+    	assertTrue("Failed to retrieve extended computer name", Kernel32.INSTANCE.GetComputerNameEx(WinBase.COMPUTER_NAME_FORMAT.ComputerNameNetBIOS, buffer, lpnSize));
+        String  actual = Native.toString(buffer);
+
+        assertEquals("Mismatched names", expected, actual);
+    }
+
     public void testWaitForSingleObject() {
 		HANDLE handle = Kernel32.INSTANCE.CreateEvent(null, false, false, null);
 
@@ -129,10 +249,10 @@ public class Kernel32Test extends TestCase {
 		// This should return successfully
 		assertEquals(WinBase.WAIT_OBJECT_0, Kernel32.INSTANCE.WaitForSingleObject(
 				 handle, 1000));
-		
+
 		// now reset it to not signaled
 		Kernel32.INSTANCE.ResetEvent(handle);
-		
+
 		// handle runs into timeout since it is not triggered
 		// WAIT_TIMEOUT = 0x00000102
 		assertEquals(WinError.WAIT_TIMEOUT, Kernel32.INSTANCE.WaitForSingleObject(
@@ -292,23 +412,6 @@ public class Kernel32Test extends TestCase {
     	assertEquals(0, lpBuffer.ullAvailExtendedVirtual.intValue());
     }
 
-    public void testGetLogicalDriveStrings() {
-    	DWORD dwSize = Kernel32.INSTANCE.GetLogicalDriveStrings(new DWORD(0), null);
-    	assertTrue(dwSize.intValue() > 0);
-    	char buf[] = new char[dwSize.intValue()];
-    	assertTrue(Kernel32.INSTANCE.GetLogicalDriveStrings(dwSize, buf).intValue() > 0);
-    }
-
-    public void testGetDiskFreeSpaceEx() {
-    	LARGE_INTEGER.ByReference lpFreeBytesAvailable = new LARGE_INTEGER.ByReference();
-    	LARGE_INTEGER.ByReference lpTotalNumberOfBytes = new LARGE_INTEGER.ByReference();
-    	LARGE_INTEGER.ByReference lpTotalNumberOfFreeBytes = new LARGE_INTEGER.ByReference();
-    	assertTrue(Kernel32.INSTANCE.GetDiskFreeSpaceEx(null,
-    			lpFreeBytesAvailable, lpTotalNumberOfBytes, lpTotalNumberOfFreeBytes));
-    	assertTrue(lpTotalNumberOfFreeBytes.getValue() > 0);
-    	assertTrue(lpTotalNumberOfFreeBytes.getValue() < lpTotalNumberOfBytes.getValue());
-    }
-
     public void testDeleteFile() {
     	String filename = Kernel32Util.getTempPath() + "\\FileDoesNotExist.jna";
     	assertFalse(Kernel32.INSTANCE.DeleteFile(filename));
@@ -321,21 +424,28 @@ public class Kernel32Test extends TestCase {
     	tmp.deleteOnExit();
 
     	FileWriter fw = new FileWriter(tmp);
-    	fw.append(expected);
-    	fw.close();
+    	try {
+    	    fw.append(expected);
+    	} finally {
+    	    fw.close();
+    	}
 
     	HANDLE hFile = Kernel32.INSTANCE.CreateFile(tmp.getAbsolutePath(), WinNT.GENERIC_READ, WinNT.FILE_SHARE_READ,
     			new WinBase.SECURITY_ATTRIBUTES(), WinNT.OPEN_EXISTING, WinNT.FILE_ATTRIBUTE_NORMAL, null);
-    	assertFalse(hFile == WinBase.INVALID_HANDLE_VALUE);
+    	assertFalse("Failed to create file handle: " + tmp, WinBase.INVALID_HANDLE_VALUE.equals(hFile));
 
-    	Memory m = new Memory(2048);
-    	IntByReference lpNumberOfBytesRead = new IntByReference(0);
-    	assertTrue(Kernel32.INSTANCE.ReadFile(hFile, m, (int) m.size(), lpNumberOfBytesRead, null));
-    	int read = lpNumberOfBytesRead.getValue();
-    	assertEquals(expected.length(), read);
-    	assertEquals(expected, new String(m.getByteArray(0, read)));
+    	try {
+            byte[] readBuffer=new byte[expected.length() + Byte.MAX_VALUE];
+        	IntByReference lpNumberOfBytesRead = new IntByReference(0);
+        	assertTrue("Failed to read from file", Kernel32.INSTANCE.ReadFile(hFile, readBuffer, readBuffer.length, lpNumberOfBytesRead, null));
 
-    	assertTrue(Kernel32.INSTANCE.CloseHandle(hFile));
+        	int read = lpNumberOfBytesRead.getValue();
+        	assertEquals("Mismatched read size", expected.length(), read);
+
+        	assertEquals("Mismatched read content", expected, new String(readBuffer, 0, read));
+    	} finally {
+    	    assertTrue("Failed to close file", Kernel32.INSTANCE.CloseHandle(hFile));
+    	}
     }
 
     public void testSetHandleInformation() throws IOException {
@@ -432,21 +542,6 @@ public class Kernel32Test extends TestCase {
         assertTrue(processInformation.dwProcessId.longValue() > 0);
     }
 
-    public void testGetEnvironmentVariable() {
-    	assertTrue(Kernel32.INSTANCE.SetEnvironmentVariable("jna-getenvironment-test", "42"));
-    	int size = Kernel32.INSTANCE.GetEnvironmentVariable("jna-getenvironment-test", null, 0);
-    	assertTrue(size == 3);
-    	char[] data = new char[size];
-    	assertEquals(size - 1, Kernel32.INSTANCE.GetEnvironmentVariable("jna-getenvironment-test", data, size));
-    	assertEquals(size - 1, Native.toString(data).length());
-    }
-
-    public void testSetEnvironmentVariable() {
-        int value = new Random().nextInt();
-        Kernel32.INSTANCE.SetEnvironmentVariable("jna-setenvironment-test", Integer.toString(value));
-        assertEquals(Integer.toString(value), Kernel32Util.getEnvironmentVariable("jna-setenvironment-test"));
-    }
-
     public void testGetSetFileTime() throws IOException {
         File tmp = File.createTempFile("testGetSetFileTime", "jna");
         tmp.deleteOnExit();
@@ -522,11 +617,11 @@ public class Kernel32Test extends TestCase {
         writer.close();
 
         final char[] buffer = new char[8];
-        
+
         DWORD len = Kernel32.INSTANCE.GetPrivateProfileString("Section", "existingKey", "DEF", buffer, new DWORD(buffer.length), tmp.getCanonicalPath());
         assertEquals(3, len.intValue());
         assertEquals("ABC", Native.toString(buffer));
-        
+
         len = Kernel32.INSTANCE.GetPrivateProfileString("Section", "missingKey", "DEF", buffer, new DWORD(buffer.length), tmp.getCanonicalPath());
         assertEquals(3, len.intValue());
         assertEquals("DEF", Native.toString(buffer));
@@ -552,7 +647,7 @@ public class Kernel32Test extends TestCase {
         assertEquals(reader.readLine(), null);
         reader.close();
     }
-    
+
     public final void testGetPrivateProfileSection() throws IOException {
         final File tmp = File.createTempFile("testGetPrivateProfileSection", ".ini");
         tmp.deleteOnExit();
@@ -614,4 +709,62 @@ public class Kernel32Test extends TestCase {
         reader.close();
     }
 
+    public final void testCreateRemoteThread() throws IOException {
+    	HANDLE hThrd = Kernel32.INSTANCE.CreateRemoteThread(null, null, 0, null, null, null, null);
+    	assertNull(hThrd);
+    	assertEquals(Kernel32.INSTANCE.GetLastError(), WinError.ERROR_INVALID_HANDLE);
+    }
+
+    public void testWriteProcessMemory() {
+    	Kernel32 kernel = Kernel32.INSTANCE;
+
+    	boolean successWrite = kernel.WriteProcessMemory(null, Pointer.NULL, Pointer.NULL, 1, null);
+    	assertFalse(successWrite);
+    	assertEquals(kernel.GetLastError(), WinError.ERROR_INVALID_HANDLE);
+
+    	ByteBuffer bufDest = ByteBuffer.allocateDirect(4);
+    	bufDest.put(new byte[]{0,1,2,3});
+    	ByteBuffer bufSrc = ByteBuffer.allocateDirect(4);
+    	bufSrc.put(new byte[]{5,10,15,20});
+    	Pointer ptrSrc = Native.getDirectBufferPointer(bufSrc);
+    	Pointer ptrDest = Native.getDirectBufferPointer(bufDest);
+
+    	HANDLE selfHandle = kernel.GetCurrentProcess();
+    	kernel.WriteProcessMemory(selfHandle, ptrDest, ptrSrc, 3, null);//Write only the first three
+
+		assertEquals(bufDest.get(0),5);
+    	assertEquals(bufDest.get(1),10);
+    	assertEquals(bufDest.get(2),15);
+    	assertEquals(bufDest.get(3),3);
+	}
+
+    public void testReadProcessMemory() {
+    	Kernel32 kernel = Kernel32.INSTANCE;
+
+    	boolean successRead = kernel.ReadProcessMemory(null, Pointer.NULL, Pointer.NULL, 1, null);
+    	assertFalse(successRead);
+    	assertEquals(kernel.GetLastError(), WinError.ERROR_INVALID_HANDLE);
+
+    	ByteBuffer bufSrc = ByteBuffer.allocateDirect(4);
+    	bufSrc.put(new byte[]{5,10,15,20});
+    	ByteBuffer bufDest = ByteBuffer.allocateDirect(4);
+    	bufDest.put(new byte[]{0,1,2,3});
+    	Pointer ptrSrc = Native.getDirectBufferPointer(bufSrc);
+    	Pointer ptrDest = Native.getDirectBufferPointer(bufDest);
+
+    	HANDLE selfHandle = kernel.GetCurrentProcess();
+    	kernel.ReadProcessMemory(selfHandle, ptrSrc, ptrDest, 3, null);//Read only the first three
+
+		assertEquals(bufDest.get(0),5);
+    	assertEquals(bufDest.get(1),10);
+    	assertEquals(bufDest.get(2),15);
+    	assertEquals(bufDest.get(3),3);
+    }
+
+    public void testVirtualQueryEx() {
+        HANDLE selfHandle = Kernel32.INSTANCE.GetCurrentProcess();
+        MEMORY_BASIC_INFORMATION mbi = new MEMORY_BASIC_INFORMATION();
+        SIZE_T bytesRead = Kernel32.INSTANCE.VirtualQueryEx(selfHandle, Pointer.NULL, mbi, new SIZE_T(mbi.size()));
+        assertTrue(bytesRead.intValue() > 0);
+    }
 }
